/*
lotto.c  Version 0.3.0  Simulate Lotto Drawing
Copyright (C) 2002-2010   dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	double odds;
    	double getlot(int select);
    	odds = getlot(select);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in lotprob.c regarding the */
/* calls to getlot.c. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rnd.h"

#define CENTURY 10436
#define FIFTYTWO 52
#define SIX 6

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s ticket\n", pgm);
   fprintf(stderr,"Where ticket is 01 02 03 04 05 06\n");
   fprintf(stderr,"Numbers 01 02 03 04 05 06 are 1 - 52\n");
   fprintf(stderr,"To see probabilities, run lotprob\n");
   fprintf(stderr,"To test quick picks,  run qpick\n");
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,max52,indx;
   unsigned char *p,*q,*r;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char fiftytwo[64];
   unsigned char ticket[64];
   unsigned char game[16];
   unsigned char trial[16];
   int tmp;
   int totmtch,won;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning,totprize,netearn;
   double winpool;
   double *prizeptr;
   double prize[16];
   double odds,sum,totbingo,chisq;
   double totprob;
   double expected;
   double totcost;
   double getlot(int select);
   double *bingoptr;
   double bingo[16];
   p = (unsigned char *) ticket;
   q = (unsigned char *) ticket + FIFTYTWO;
   while (p < q)
      *p++ = (unsigned char) '\0';
   if (argc != 7) putstx(*argv);
   gameptr = (unsigned char *) game;
   i = 1;
   while (i <= 6)
      {
      indx = atoi(*(argv+i));
      if (indx < 1 || indx > FIFTYTWO)
	 {
	 fprintf(stderr,"lotto: ticket number %s "
	    "is invalid\n",
            *(argv+i));
	 putstx(*argv);
	 } /* if invalid number */
      p = (unsigned char *) ticket + indx;
      if (*p != 255)
         *p = (unsigned char) 255;
      else
	 {
	 fprintf(stderr,"lotto: ticket number %s "
	    "is a duplicate\n",
            *(argv+i));
	 putstx(*argv);
	 } /* duplicate number */
      *gameptr++ = (unsigned char) indx;
      i++;
      } /* for each number on ticket */
   /* two games for $1.00 */
   /* twice/week for 100 years */
   /* 2 * 52 * 2 * 100 = 20800 */
   /* Based on Julian days, it computes to 20872 */
   /* (((2488434.5 - 2451910.5) / 7) * 2) * 2) = 20872 */
   maxplay = CENTURY << 1;
   sd = (unsigned char *) rndinit();
   dblplay = (double) maxplay;
   totcost = (double) CENTURY;
   winpool = totcost * 0.50;
   prizeptr = (double *) prize;
   *prizeptr++ = 2000000.00;
   *prizeptr++ = 1000.00;
   *prizeptr++ = 33.00;
   *prizeptr   = 3.00;
   printf("Lotto ticket:  ");
   gameptr = (unsigned char *) game;
   i = SIX;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* 6 numbers on ticket */
   putchar('\n');
   putchar('\n');
   printf("Match      Wins          Prize    "
      "   Expected           Odds\n");
   bingoptr = (double *) bingo;
   i = 4;
   while (i--) *bingoptr++ = 0.0;
   bingoptr = (double *) bingo;
   totplay = maxplay;
   while (totplay--)
      {
      p = (unsigned char *) fiftytwo;
      q = (unsigned char *) fiftytwo + FIFTYTWO;
      i = 0;
      while (p < q)
         *p++ = (unsigned char) ++i;
      *p = (unsigned char) '\0';
      totmtch = 0;
      p = (unsigned char *) fiftytwo;
      q = (unsigned char *) fiftytwo + FIFTYTWO - 1;
      r = (unsigned char *) ticket;
      trialptr = (unsigned char *) trial;
      max52 = FIFTYTWO;
      i = SIX;
      while (i--)
         {
         indx = rndnum(max52,sd);
         tmp = (int) *(p+indx);
         *trialptr++ = (unsigned char) tmp;
         if (*(r+tmp) == 255) totmtch++;
         *(p+indx) = *q;
         *q-- = '\0';
         max52--;
         } /* pick 6 balls without replacement */
      if (totmtch == SIX)
         won = 0;
      else if (totmtch == 5)
         won = 1;
      else if (totmtch == 4)
         won = 2;
      else if (totmtch == 3)
         won = 3;
      else won = 4;
      *(bingoptr+won) += 1.0;
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   totprize = 0.0;
   /* Grand Prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(6);
   expected = dblplay / prob;
   printf("  6   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Second Prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(5);
   expected = dblplay / prob;
   printf("  5   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Third Prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(4);
   expected = dblplay / prob;
   printf("  4   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Fourth Prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(3);
   expected = dblplay / prob;
   printf("  3   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   bingoptr = (double *) bingo;
   sum = totbingo = totprob = 0.0;
   odds = getlot(6);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(5);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(4);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(3);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;

   sum += ((dblplay - totbingo)
      * (dblplay - totbingo)
      / (1.0 - totprob));

   expected = dblplay / (1.0 / (1.0 - totprob));
   printf("  0   %9.0f  %13.2f  %13.2f   %21.10f\n",
      (dblplay - totbingo),
      0.0,
      expected,
      (1.0 / (1.0 - totprob)));
   printf("        -------      ---------\n");
   printf("Total %9.0f  %13.2f\n\n",
      dblplay, totprize);
   printf("Total Cost       %13.2f\n\n",
      totcost);
   netearn = totprize - totcost;
   printf("Net Income       %13.2f\n\n",
      netearn);
   chisq = (sum / dblplay) - dblplay;
   printf("Chi-square %f\n", chisq);
   printf("Range at 95 percent "
      "is 0.484419 to 11.1433\n");
   free(sd);
   return(0);
   } /* main */
